@@ -13,15 +13,25 @@
 ##############################################################################
 """XMLRPC testing helpers for Zope 3.
 
-$Id: xmlrpc.py 110724 2010-04-11 00:04:11Z tseaver $
+$Id: xmlrpc.py 116141 2010-09-02 15:05:04Z patricks $
 """
 
+import httplib
 import StringIO
-import xmlrpclib 
+import xmlrpclib
 
 from zope.app.testing.functional import HTTPCaller
 
 
+class FakeSocket(object):
+
+    def __init__(self, data):
+        self.data = data
+
+    def makefile(self, mode, bufsize=None):
+        return StringIO.StringIO(self.data)
+
+
 class ZopeTestTransport(xmlrpclib.Transport):
     """xmlrpclib transport that delegates to
     zope.app.testing.functional.HTTPCaller.
@@ -58,9 +68,9 @@ class ZopeTestTransport(xmlrpclib.Transport):
                 errcode, errmsg,
                 headers
                 )
-
-        return self._parse_response(
-            StringIO.StringIO(response.getBody()), sock=None)
+        res = httplib.HTTPResponse(FakeSocket(response.getBody()))
+        res.begin()
+        return self.parse_response(res)
 
 
 def ServerProxy(uri, transport=None, encoding=None,
